@@ -160,6 +160,121 @@ test_that("convert to vector works for tibble", {
   )
 })
 
+test_that("convert to vector works for nanoarrow_vctr()", {
+  array <- as_nanoarrow_array(c("one", "two", "three"))
+
+  # Check implicit/inferred nanoarrow_vctr() schema
+  vctr <- convert_array(array, nanoarrow_vctr())
+  expect_s3_class(vctr, "nanoarrow_vctr")
+  expect_length(vctr, 3)
+  schema <- infer_nanoarrow_schema(vctr)
+  expect_identical(schema$format, "u")
+
+  # Check with explicit schema of the correct type
+  vctr <- convert_array(array, nanoarrow_vctr(na_string()))
+  expect_s3_class(vctr, "nanoarrow_vctr")
+  expect_length(vctr, 3)
+  schema <- infer_nanoarrow_schema(vctr)
+  expect_identical(schema$format, "u")
+
+  # Check conversion of a struct array
+  df <- data.frame(x = c("one", "two", "three"))
+  array <- as_nanoarrow_array(df)
+
+  vctr <- convert_array(array, nanoarrow_vctr())
+  expect_s3_class(vctr, "nanoarrow_vctr")
+  expect_length(vctr, 3)
+  schema <- infer_nanoarrow_schema(vctr)
+  expect_identical(schema$format, "+s")
+
+  vctr <- convert_array(array, nanoarrow_vctr(na_struct(list(x = na_string()))))
+  expect_s3_class(vctr, "nanoarrow_vctr")
+  expect_length(vctr, 3)
+  schema <- infer_nanoarrow_schema(vctr)
+  expect_identical(schema$format, "+s")
+})
+
+test_that("batched convert to vector works for nanoarrow_vctr()", {
+  empty_stream <- basic_array_stream(list(), schema = na_string())
+  empty_vctr <- convert_array_stream(empty_stream, nanoarrow_vctr())
+  expect_length(empty_vctr, 0)
+  expect_identical(infer_nanoarrow_schema(empty_vctr)$format, "u")
+
+  stream1 <- basic_array_stream(list(c("one", "two", "three")))
+  vctr1 <- convert_array_stream(stream1, nanoarrow_vctr())
+  expect_length(vctr1, 3)
+
+  stream2 <- basic_array_stream(
+    list(c("one", "two", "three"), c("four", "five", "six", "seven"))
+  )
+  vctr2 <- convert_array_stream(stream2, nanoarrow_vctr())
+  expect_length(vctr2, 7)
+  expect_identical(
+    convert_array_stream(as_nanoarrow_array_stream(vctr2)),
+    c("one", "two", "three", "four", "five", "six", "seven")
+  )
+})
+
+test_that("convert to vector works for data.frame(nanoarrow_vctr())", {
+  array <- as_nanoarrow_array(data.frame(x = 1:5))
+  df_vctr <- convert_array(array, data.frame(x = nanoarrow_vctr()))
+  expect_s3_class(df_vctr$x, "nanoarrow_vctr")
+  expect_identical(
+    convert_array_stream(as_nanoarrow_array_stream(df_vctr$x)),
+    1:5
+  )
+})
+
+test_that("convert to vector works for list_of(nanoarrow_vctr())", {
+  skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
+
+  array <- as_nanoarrow_array(
+    list(1:5, 6:10, NULL, 11:13),
+    schema = na_list(na_int32())
+  )
+
+  list_vctr <- convert_array(array, vctrs::list_of(nanoarrow_vctr()))
+
+  # Each item in the list should be a vctr with one chunk that is a slice
+  # of the original array
+  expect_s3_class(list_vctr[[1]], "nanoarrow_vctr")
+  vctr_array <- attr(list_vctr[[1]], "chunks")[[1]]
+  expect_identical(vctr_array$offset, 0L)
+  expect_identical(vctr_array$length, 5L)
+  expect_identical(convert_buffer(vctr_array$buffers[[2]]), 1:5)
+
+  expect_s3_class(list_vctr[[2]], "nanoarrow_vctr")
+  vctr_array <- attr(list_vctr[[2]], "chunks")[[1]]
+  expect_identical(vctr_array$offset, 5L)
+  expect_identical(vctr_array$length, 5L)
+  expect_identical(convert_buffer(vctr_array$buffers[[2]]), 1:10)
+
+  expect_null(list_vctr[[3]])
+
+  expect_s3_class(list_vctr[[4]], "nanoarrow_vctr")
+  vctr_array <- attr(list_vctr[[4]], "chunks")[[1]]
+  expect_identical(vctr_array$offset, 10L)
+  expect_identical(vctr_array$length, 3L)
+  expect_identical(convert_buffer(vctr_array$buffers[[2]]), 1:13)
+})
+
+test_that("batched convert to vector works for nanoarrow_vctr() keeps subclass", {
+  vctr_ptype <- nanoarrow_vctr(subclass = "some_subclass")
+
+  empty_stream <- basic_array_stream(list(), schema = na_string())
+  empty_vctr <- convert_array_stream(empty_stream, vctr_ptype)
+  expect_s3_class(empty_vctr, "some_subclass")
+
+  stream1 <- basic_array_stream(list(c("")))
+  vctr1 <- convert_array_stream(stream1, vctr_ptype)
+  expect_s3_class(vctr1, "some_subclass")
+
+  stream2 <- basic_array_stream(list(c(""), c("")))
+  vctr2 <- convert_array_stream(stream2, vctr_ptype)
+  expect_s3_class(vctr2, "some_subclass")
+})
+
 test_that("convert to vector works for struct-style vectors", {
   array <- as_nanoarrow_array(as.POSIXlt("2021-01-01", tz = "America/Halifax"))
   expect_identical(
@@ -445,6 +560,7 @@ test_that("convert to vector works for valid double()", {
     uint32 = arrow::uint32(),
     int64 = arrow::int64(),
     uint64 = arrow::uint64(),
+    float16 = arrow::float16(),
     float32 = arrow::float32(),
     float64 = arrow::float64()
   )
@@ -666,6 +782,20 @@ test_that("convert to vector works for character()", {
   )
 })
 
+test_that("convert to vector works for string_view -> character()", {
+  array <- as_nanoarrow_array(letters, schema = na_string_view())
+  expect_identical(
+    convert_array(array, character()),
+    letters
+  )
+
+  array_with_nulls <- as_nanoarrow_array(c(letters, NA), schema = na_string_view())
+  expect_identical(
+    convert_array(array_with_nulls, character()),
+    c(letters, NA)
+  )
+})
+
 test_that("convert to vector works for null -> character()", {
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
@@ -778,6 +908,22 @@ test_that("convert to vector works for blob::blob()", {
   )
 })
 
+test_that("convert to vector works for binary_view -> blob::blob()", {
+  skip_if_not_installed("blob")
+
+  array <- as_nanoarrow_array(letters, schema = na_binary_view())
+  expect_identical(
+    convert_array(array, blob::blob()),
+    blob::as_blob(lapply(letters, charToRaw))
+  )
+
+  array_with_nulls <- as_nanoarrow_array(c(letters, NA), schema = na_binary_view())
+  expect_identical(
+    convert_array(array_with_nulls, blob::blob()),
+    blob::as_blob(c(lapply(letters, charToRaw), list(NULL)))
+  )
+})
+
 test_that("convert to vector works for null -> blob::blob()", {
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
